@@ -20,6 +20,11 @@ from inspect import isclass, stack
 from colorit import color_front, bold
 
 
+# Enable to see JIT/Numba debug output
+# import os
+# os.environ["NUMBA_PARALLEL_DIAGNOSTICS"] = "4"
+
+
 class Debug:
     """ Debug class. """
 
@@ -27,23 +32,27 @@ class Debug:
     Whitelist = [
         "About_Dialog",
         "Assert_Dialog",
-        "BiotSavart",
+        "BiotSavart_JIT",
+        "BiotSavart_CUDA",
         "CalculationThread",
         # "Config",
+        "Constants",
+        "Constraint",
         "Display_Widget",
-        "Field",
+        # "Field",
+        "Field_Widget",
         "Groupbox",
         "GUI",
         "HLine",
         "IconLabel",
         "Menu",
-        "Metric",
+        # "Metric",
         "Metric_Widget",
-        "Model",
-        "ModelAccess",
+        # "Model",
+        # "ModelAccess",
         "Perspective_Widget",
-        "SamplingVolume",
-        "SamplingVolume_Widget",
+        # "SamplingVolume",
+        # "SamplingVolume_Widget",
         "SidebarLeft",
         "SidebarRight",
         "SliderFloat",
@@ -51,8 +60,8 @@ class Debug:
         "Table",
         "Usage_Dialog",
         "Version",
-        "VispyCanvas",
-        "Wire",
+        # "VispyCanvas",
+        # "Wire",
         "Wire_Widget"
     ]
 
@@ -66,10 +75,10 @@ class Debug:
         @param force: Enable to override whitelist
         """
         if isclass(obj):
-            # Called from within class method, i.e. Debug(cls, ...)
+            # Called from within class method, i.e. Debug(self, ...)
             name = obj.__name__
         else:
-            # Called from within instance method, i.e. Debug(self, ...)
+            # Called from within instance / static method
             name = type(obj).__name__
 
         if not force:
@@ -96,6 +105,10 @@ class Debug:
                 func_str = "\tCalculationThread"
             hierarchy += func_str + "/"
 
+        if isinstance(color, str):
+            # Convert hex color string ("#abcdef") to RGB tuple
+            color = tuple(int(color[i:i+2], 16) for i in (1, 3, 5))
+
         if color == (0, 0, 0):
             # Allow a terminal use its own foreground color
             color_text = bold(name) + text
