@@ -21,7 +21,7 @@ import time
 import atexit
 import datetime
 import qtawesome as qta
-from PyQt5.QtCore import Qt, QThread, pyqtSignal
+from PyQt5.QtCore import Qt, QThread, pyqtSignal, QLocale
 from PyQt5.QtWidgets import QMainWindow, QSplitter, QFileDialog, QDesktopWidget
 from magneticalc.CalculationThread import CalculationThread
 from magneticalc.Debug import Debug
@@ -30,6 +30,7 @@ from magneticalc.Model import Model
 from magneticalc.SidebarLeft import SidebarLeft
 from magneticalc.SidebarRight import SidebarRight
 from magneticalc.Statusbar import Statusbar
+from magneticalc.Theme import Theme
 from magneticalc.Version import Version
 from magneticalc.VispyCanvas import VispyCanvas
 
@@ -51,6 +52,8 @@ class GUI(QMainWindow):
 
         Debug(self, ": Init")
 
+        self.locale = QLocale(QLocale.English)
+
         self.config = config
 
         self.set_window()
@@ -63,18 +66,16 @@ class GUI(QMainWindow):
         atexit.register(self.quit)
 
         # Create the model first, as the following objects will access it (each widget acts as view *and* controller)
-        self.model = Model(
-            # This callback is needed to clear the metric labels after the metric became invalid
-            on_metric_invalidated=self.on_metric_invalidated
-        )
+        self.model = Model(self)
 
         # Create the left and right sidebar
-        # Note: These create the wire, sampling volume and metric widgets, each populating the model from configuration
+        # Note: These create the wire, sampling volume, field and metric widgets, each populating the model from config.
         self.sidebar_left = SidebarLeft(self)
         self.sidebar_right = SidebarRight(self)
 
         # Create the VisPy canvas (our 3D scene) and statusbar
         self.vispy_canvas = VispyCanvas(self)
+        self.vispy_canvas.native.setFocusPolicy(Qt.NoFocus)  # Don't let VisPy gain control -- handle all events in GUI
         self.statusbar = Statusbar(self)
 
         # Insert left sidebar, VisPy canvas and right sidebar into main layout.
@@ -103,12 +104,14 @@ class GUI(QMainWindow):
         """
         if self.calculation_thread is not None:
             if self.calculation_thread.isRunning():
-                Debug(self, ".redraw(): Skipped because calculation is in progress", color=(0, 0, 255))
+                Debug(self, ".redraw(): Skipped because calculation is in progress", color=Theme.PrimaryColor)
                 return
             else:
                 Debug(self, ".redraw(): WARNING: Setting calculation thread to None", color=(255, 0, 0))
                 self.calculation_thread = None
 
+        self.sidebar_right.display_widget.set_enabled(self.model.field.is_valid())
+
         self.vispy_canvas.redraw()
 
     # ------------------------------------------------------------------------------------------------------------------
@@ -151,7 +154,10 @@ class GUI(QMainWindow):
                 self.calculation_thread = None
                 Debug(self, f".on_calculation_exited(): Success (took {calculation_time:.2f} s)", color=(0, 128, 0))
         else:
-            Debug(self, f".on_calculation_exited(): Interrupted after {calculation_time:.2f} s", color=(0, 0, 255))
+            Debug(
+                self,
+                f".on_calculation_exited(): Interrupted after {calculation_time:.2f} s", color=Theme.PrimaryColor
+            )
 
         # Note: For some reason, most of the time we need an additional ("final-final") re-draw here
         self.redraw()
@@ -167,11 +173,11 @@ class GUI(QMainWindow):
             return
 
         if self.calculation_thread.isRunning():
-            Debug(self, ".interrupt_calculation(): Requesting interruption", color=(0, 0, 255))
+            Debug(self, ".interrupt_calculation(): Requesting interruption", color=Theme.PrimaryColor)
             self.calculation_thread.requestInterruption()
 
-            if self.calculation_thread.wait(1000):
-                Debug(self, ".interrupt_calculation(): Exited gracefully", color=(0, 0, 255))
+            if self.calculation_thread.wait(3000):
+                Debug(self, ".interrupt_calculation(): Exited gracefully", color=Theme.PrimaryColor)
             else:
                 Debug(self, ".interrupt_calculation(): WARNING: Terminating ungracefully", color=(255, 0, 0))
                 self.calculation_thread.terminate()
@@ -183,15 +189,6 @@ class GUI(QMainWindow):
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def on_metric_invalidated(self):
-        """
-        Needed to clear the metric labels after the metric became invalid.
-        """
-        self.sidebar_right.metric_widget.invalidate_labels()
-        self.vispy_canvas.delete_field_labels()
-
-    # ------------------------------------------------------------------------------------------------------------------
-
     def set_window(self):
         """
         Sets the basic window properties.
@@ -199,7 +196,7 @@ class GUI(QMainWindow):
 
         # Set window title and icon
         self.setWindowTitle(Version.String)
-        self.setWindowIcon(qta.icon("ei.magnet"))
+        self.setWindowIcon(qta.icon("ei.magnet", color=Theme.PrimaryColor))
 
         # Adjust window dimensions to desktop dimensions
         screen = QDesktopWidget().screenGeometry()
@@ -242,9 +239,31 @@ class GUI(QMainWindow):
 
         @param event: Key press event
         """
-        if event.key() == Qt.Key_Escape:
+        if event.key() == Qt.Key_F2:
+
+            # Focus the the wire base points table
+            self.sidebar_left.wire_widget.table.setFocus()
+
+        elif event.key() == Qt.Key_F5:
+
+            # Focus the main window (make sure to un-focus the wire base points table)
             self.setFocus()
 
+            # (Re-)Start calculation
+            self.recalculate()
+
+        elif event.key() == Qt.Key_Escape:
+
+            if self.sidebar_left.wire_widget.table.hasFocus():
+                # Focus the main window, thus un-focusing the wire base points table
+                self.setFocus()
+            else:
+                # Stop any running calculation
+                if self.calculation_thread is not None:
+                    if self.calculation_thread.isRunning():
+                        # Cancel the running calculation
+                        self.interrupt_calculation()
+
     def file_save(self):
         """
         Saves the currently displayed scene to PNG file.
