@@ -27,12 +27,15 @@ class IconLabel(QWidget):
     IconSize = QSize(16, 16)
     HorizontalSpacing = 1
 
-    def __init__(self, qta_id, text):
+    def __init__(self, qta_id, text, icon_color="#12344a", label_color="#12344a", final_stretch=True):
         """
         Initializes the icon label.
 
         @param qta_id: QtAwesome icon id
         @param text: Label text
+        @param icon_color: Icon color (string)
+        @param label_color: Label color (string)
+        @param final_stretch: Enable to add a final stretch
         """
         # noinspection PyArgumentList
         QWidget.__init__(self)
@@ -43,17 +46,19 @@ class IconLabel(QWidget):
 
         if qta_id is not None:
             icon = QLabel()
-            icon.setPixmap(qta.icon(qta_id).pixmap(self.IconSize))
+            icon.setPixmap(qta.icon(qta_id, color=icon_color).pixmap(self.IconSize))
             self.layout.addWidget(icon, alignment=Qt.AlignVCenter)
             self.layout.addSpacing(self.HorizontalSpacing)
 
         label = QLabel(text)
-        label.setStyleSheet("""
+        label.setStyleSheet(f"""
+            color: {label_color};
             font-weight: bold;
         """)
         self.layout.addWidget(label, alignment=Qt.AlignVCenter)
 
-        self.layout.addStretch()
+        if final_stretch:
+            self.layout.addStretch()
 
     # noinspection PyPep8Naming
     def addWidget(self, widget):
