@@ -17,9 +17,10 @@
 #  OR IN CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 
 import qtawesome as qta
+from multiprocessing import cpu_count
 from PyQt5.QtCore import Qt
 from PyQt5.QtWidgets import QWidget, QHBoxLayout, QPushButton, QCheckBox, QComboBox, QProgressBar, QLabel
-from multiprocessing import cpu_count
+from magneticalc.Theme import Theme
 
 
 class Statusbar:
@@ -33,14 +34,22 @@ class Statusbar:
         """
         self.gui = gui
 
+        # Start button
+        self.start_button = QPushButton(qta.icon("fa.play-circle"), "", self.gui)
+        self.start_button.setText(" F5 ")
+        self.start_button.setStyleSheet(f"padding: 3px; font-size: 13px;")
+        self.start_button.clicked.connect(self.start)
+
         # Cancel button
-        self.cancel_button = QPushButton(qta.icon("fa.window-close"), "", self.gui)
+        self.cancel_button = QPushButton(qta.icon("fa.stop-circle"), "", self.gui)
+        self.cancel_button.setText("ESC")
+        self.cancel_button.setStyleSheet(f"padding: 3px; font-size: 13px;")
         self.cancel_button.clicked.connect(self.cancel)
 
         # Auto-calculation checkbox
         self.auto_calculation_checkbox = QCheckBox("Auto-Calculation")
         self.auto_calculation_checkbox.setChecked(self.gui.config.get_bool("auto_calculation"))
-        self.auto_calculation_checkbox.stateChanged.connect(self.auto_calculation_changed)
+        self.auto_calculation_checkbox.stateChanged.connect(self._auto_calculation_changed)
 
         # Number-of-cores combobox
         self.cores_combobox = QComboBox()
@@ -71,6 +80,8 @@ class Statusbar:
 
         # Populate statusbar layout
         layout = QHBoxLayout()
+        layout.addWidget(self.start_button, alignment=Qt.AlignVCenter)
+        layout.addSpacing(4)
         layout.addWidget(self.cancel_button, alignment=Qt.AlignVCenter)
         layout.addSpacing(4)
         layout.addWidget(self.auto_calculation_checkbox, alignment=Qt.AlignVCenter)
@@ -82,21 +93,35 @@ class Statusbar:
         layout.addWidget(self.label, alignment=Qt.AlignVCenter)
         layout.setContentsMargins(8, 2, 10, 2)
 
-        # noinspection PyArgumentList
         container_widget = QWidget()
         container_widget.setLayout(layout)
         gui.statusBar().addPermanentWidget(container_widget, stretch=10)
 
         gui.statusBar().setSizeGripEnabled(False)
 
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def _auto_calculation_changed(self):
+        """
+        Handles changed auto-calculation setting.
+        """
+        self.gui.config.set_bool("auto_calculation", self.auto_calculation_checkbox.isChecked())
+        if self.auto_calculation_checkbox.isChecked():
+            if not self.gui.model.is_valid():
+                self.gui.recalculate()
+
+    # ------------------------------------------------------------------------------------------------------------------
+
     def arm(self):
         """
         "Arms" the statusbar before calculation.
         """
+        self.start_button.setEnabled(False)
         self.cancel_button.setEnabled(True)
         self.auto_calculation_checkbox.setEnabled(False)
         self.cores_combobox.setEnabled(False)
         self.progressbar.setValue(0)
+        self.set_progressbar_color(Theme.PrimaryColor)
 
     def disarm(self, success):
         """
@@ -104,14 +129,23 @@ class Statusbar:
 
         @param success: Reflects calculation success or failure
         """
+        self.start_button.setEnabled(True)
         self.cancel_button.setEnabled(False)
         self.auto_calculation_checkbox.setEnabled(True)
         self.cores_combobox.setEnabled(True)
         self.text("Ready." if success else "Canceled!")
+        self.set_progressbar_color("#2e7d32" if success else "#c62828")
 
         if success:
             self.progressbar.setValue(100)
 
+    def start(self):
+        """
+        Starts the calculation.
+        """
+        self.gui.recalculate()
+        self.disarm(True)
+
     def cancel(self):
         """
         Cancels the ongoing calculation.
@@ -127,11 +161,28 @@ class Statusbar:
         """
         self.label.setText(text)
 
-    def auto_calculation_changed(self):
+    def set_progressbar_color(self, color):
         """
-        Handles changed auto-calculation setting.
+        Sets the progressbar color.
+
+        @param color: Color (string)
         """
-        self.gui.config.set_bool("auto_calculation", self.auto_calculation_checkbox.isChecked())
-        if self.auto_calculation_checkbox.isChecked():
-            if not self.gui.model.is_valid():
-                self.gui.recalculate()
+        self.progressbar.setStyleSheet(
+            f"""
+            QProgressBar
+            {{
+                border: 1px solid #888888;
+                border-radius: 5px;
+                color: #ffffff;
+                text-align: center;
+                padding: 0px;
+                height: 23px;
+                background-color: #111111;
+                font-size: 13px;
+            }}
+            QProgressBar::chunk {{
+                padding: 0px;
+                background-color: {color};
+                text-align: center;
+            }}"""
+        )
