@@ -20,6 +20,7 @@ import numpy as np
 from PyQt5.QtCore import QThread
 from magneticalc.Assert_Dialog import Assert_Dialog
 from magneticalc.Debug import Debug
+from magneticalc.Theme import Theme
 
 
 class Wire:
@@ -121,11 +122,11 @@ class Wire:
         "id": "Solenoid: 4 circular loops",
         "points": [
             [
-                i / 64 - 1 / 2,
-                -np.cos(2 * np.pi * i / 16) / 2,
-                +np.sin(2 * np.pi * i / 16) / 2
+                i / 128 - 1 / 2,
+                -np.cos(2 * np.pi * i / 32) / 2,
+                +np.sin(2 * np.pi * i / 32) / 2
             ]
-            for i in range(64)
+            for i in range(128)
         ]
     }
 
@@ -134,11 +135,11 @@ class Wire:
         "id": "Solenoid: 8 circular loops",
         "points": [
             [
-                i / 128 - 1 / 2,
-                -np.cos(2 * np.pi * i / 16) / 2,
-                +np.sin(2 * np.pi * i / 16) / 2
+                i / 256 - 1 / 2,
+                -np.cos(2 * np.pi * i / 32) / 2,
+                +np.sin(2 * np.pi * i / 32) / 2
             ]
-            for i in range(128)
+            for i in range(256)
         ]
     }
 
@@ -147,19 +148,19 @@ class Wire:
         "id": "Compensated Solenoid: 2x 4 circular loops",
         "points": [
             [
-                i / 64 - 1 - 0.125 / 2,
-                -np.cos(+2 * np.pi * i / 16) / 2,
-                +np.sin(+2 * np.pi * i / 16) / 2
+                i / 256 - 1/2,
+                -np.cos(+2 * np.pi * i / 32) / 2,
+                +np.sin(+2 * np.pi * i / 32) / 2
             ]
-            for i in range(64)
+            for i in range(128)
         ] +
         [
             [
-                i / 64 + 0.125 / 2,
-                -np.cos(-2 * np.pi * (i + 1) / 16) / 2,
-                +np.sin(-2 * np.pi * (i + 1) / 16) / 2
+                i / 256,
+                -np.cos(-2 * np.pi * (i + 1) / 32) / 2,
+                +np.sin(-2 * np.pi * (i + 1) / 32) / 2
             ]
-            for i in range(64)
+            for i in range(128)
         ]
     }
 
@@ -168,19 +169,19 @@ class Wire:
         "id": "Compensated Solenoid: 2x 8 circular loops",
         "points": [
             [
-                i / 128 - 1.125,
-                -np.cos(+2 * np.pi * i / 16) / 2,
-                +np.sin(+2 * np.pi * i / 16) / 2
+                i / 512 - 1/2,
+                -np.cos(+2 * np.pi * i / 32) / 2,
+                +np.sin(+2 * np.pi * i / 32) / 2
             ]
-            for i in range(128)
+            for i in range(256)
         ] +
         [
             [
-                i / 128 + 0.125,
-                -np.cos(-2 * np.pi * (i + 1) / 16) / 2,
-                +np.sin(-2 * np.pi * (i + 1) / 16) / 2
+                i / 512,
+                -np.cos(-2 * np.pi * (i + 1) / 32) / 2,
+                +np.sin(-2 * np.pi * (i + 1) / 32) / 2
             ]
-            for i in range(128)
+            for i in range(256)
         ]
     }
 
@@ -217,7 +218,7 @@ class Wire:
         """
         A 3D piecewise linear curve with some DC current associated with it.
 
-        @param points: List of 3D coordinates (see presets)
+        @param points: Ordered list of 3D coordinates (see presets)
         @param stretch: XYZ stretch transform factors (3D point)
         @param rotational_symmetry: Dictionary for rotational symmetry transform
         @param slicer_limit: Slicer limit
@@ -227,34 +228,33 @@ class Wire:
 
         self._points_base = np.array(points)
 
-        print(self.CompensatedSolenoidCircularLoops8["points"])
-
-        """
         # Note: This is my playground for creating new wire presets!
-        self._points_base = np.array(
-            [
+        override_base = False
+        if override_base:
+            self._points_base = np.array(
                 [
-                    i / 128 - 1.125,
-                    -np.cos(+2 * np.pi * i / 16) / 2,
-                    +np.sin(+2 * np.pi * i / 16) / 2
-                ]
-                for i in range(128)
-            ] +
-            [
+                    [
+                        i / 128 - 1.125,
+                        -np.cos(+2 * np.pi * i / 16) / 2,
+                        +np.sin(+2 * np.pi * i / 16) / 2
+                    ]
+                    for i in range(128)
+                ] +
                 [
-                    i / 128 + 0.125,
-                    -np.cos(-2 * np.pi * (i + 1) / 16) / 2,
-                    +np.sin(-2 * np.pi * (i + 1) / 16) / 2
+                    [
+                        i / 128 + 0.125,
+                        -np.cos(-2 * np.pi * (i + 1) / 16) / 2,
+                        +np.sin(-2 * np.pi * (i + 1) / 16) / 2
+                    ]
+                    for i in range(128)
                 ]
-                for i in range(128)
-            ]
-        )
-        """
+            )
 
         self._slicer_limit = slicer_limit
         self._dc = dc
 
         self._points_sliced = None
+        self._length = None
 
         self._points_transformed = self._points_base.copy()
 
@@ -278,17 +278,10 @@ class Wire:
         Debug(self, ".invalidate()", color=(128, 0, 0))
 
         self._points_sliced = None
+        self._length = None
 
     # ------------------------------------------------------------------------------------------------------------------
 
-    def get_dc(self):
-        """
-        Returns DC value.
-
-        @return: DC value
-        """
-        return self._dc
-
     def get_bounds(self):
         """
         Returns this curve's bounding box.
@@ -300,26 +293,11 @@ class Wire:
         bounds_max = [max(axes[0]), max(axes[1]), max(axes[2])]
         return bounds_min, bounds_max
 
-    def get_elements(self):
-        """
-        Returns this curve's elements, i.e. a list of segment center points and directions.
-
-        @return: [(element_center, element_direction), ...]
-        """
-        Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
-
-        result = []
-        for i in range(len(self._points_sliced) - 1):
-            element_direction = np.array(self._points_sliced[i + 1]) - np.array(self._points_sliced[i])
-            element_center = self._points_sliced[i] + element_direction / 2
-            result.append((element_center, element_direction))
-        return result
-
     def get_points_base(self):
         """
         Returns this wire's base points.
 
-        @return: List of 3D points
+        @return: Ordered list of 3D points
         """
         return self._points_base
 
@@ -327,7 +305,7 @@ class Wire:
         """
         Returns this wire's transformed points.
 
-        @return: List of 3D points
+        @return: Ordered list of 3D points
         """
         return self._points_transformed
 
@@ -335,12 +313,45 @@ class Wire:
         """
         Returns this wire's points after slicing.
 
-        @return: List of 3D points
+        @return: Ordered list of 3D points
         """
         Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
 
         return self._points_sliced
 
+    def get_elements(self):
+        """
+        Returns this curve's elements, i.e. an ordered list of segment center points and directions.
+
+        @return: [(element_center, element_direction), ...]
+        """
+        Assert_Dialog(self.is_valid(), "Accessing invalidated wire")
+
+        result = []
+
+        for i in range(len(self._points_sliced) - 1):
+            element_direction = np.array(self._points_sliced[i + 1]) - np.array(self._points_sliced[i])
+            element_center = self._points_sliced[i] + element_direction / 2
+            result.append((element_center, element_direction))
+
+        return np.array(result)
+
+    def get_dc(self):
+        """
+        Returns DC value.
+
+        @return: DC value
+        """
+        return self._dc
+
+    def get_length(self):
+        """
+        Returns this curve's length.
+
+        @return: Length (float)
+        """
+        return self._length
+
     # ------------------------------------------------------------------------------------------------------------------
 
     def _set_stretch(self, stretch):
@@ -361,45 +372,6 @@ class Wire:
 
         self._points_transformed = axes.transpose()
 
-    def recalculate(self, progress_callback):
-        """
-        Slices wire segments into smaller ones until segment lengths equal or undershoot slicer limit.
-
-        @param progress_callback: Progress callback
-        @return: True if successful, False if interrupted
-        """
-        Debug(self, ".recalculate()", color=(0, 128, 0))
-
-        points_sliced = []
-
-        for i in range(len(self.get_points_transformed()) - 1):
-
-            # Calculate direction and length of wire segment
-            segment_direction = np.array(self.get_points_transformed()[i + 1] - self.get_points_transformed()[i])
-            segment_length = np.linalg.norm(segment_direction)
-
-            # Calculate required number of slices (subdivisions) and perform linear interpolation
-            slices = np.ceil(segment_length / self._slicer_limit).astype(int)
-            linear = np.linspace(0, 1, slices, endpoint=False)
-            points_sliced += [self.get_points_transformed()[i] + segment_direction * j for j in linear]
-
-            # Signal progress update, handle interrupt (every 16 iterations to keep overhead low)
-            if i & 0xf == 0:
-                progress_callback(100 * (i + 1) / (len(self.get_points_transformed()) - 1))
-
-                if QThread.currentThread().isInterruptionRequested():
-                    Debug(self, ".recalculate(): Interruption requested, exiting now", color=(0, 0, 255))
-                    return False
-
-        # Close the loop
-        points_sliced.append(self.get_points_transformed()[-1])
-
-        self._points_sliced = np.array(points_sliced)
-
-        return True
-
-    # ------------------------------------------------------------------------------------------------------------------
-
     def _set_rotational_symmetry(self, parameters):
         """
         This transformation replicates and rotates this curve `count` times about an `axis` with radius `radius`.
@@ -431,3 +403,45 @@ class Wire:
         self._points_transformed = np.array(axes).transpose()
 
         return self
+
+    # ------------------------------------------------------------------------------------------------------------------
+
+    def recalculate(self, progress_callback):
+        """
+        Slices wire segments into smaller ones until segment lengths equal or undershoot slicer limit.
+
+        @param progress_callback: Progress callback
+        @return: True if successful, False if interrupted
+        """
+        Debug(self, ".recalculate()", color=(0, 128, 0))
+
+        points_sliced = []
+        length = 0
+
+        for i in range(len(self.get_points_transformed()) - 1):
+
+            # Calculate direction and length of wire segment
+            segment_direction = np.array(self.get_points_transformed()[i + 1] - self.get_points_transformed()[i])
+            segment_length = np.linalg.norm(segment_direction)
+            length += segment_length
+
+            # Calculate required number of slices (subdivisions) and perform linear interpolation
+            slices = np.ceil(segment_length / self._slicer_limit).astype(int)
+            linear = np.linspace(0, 1, slices, endpoint=False)
+            points_sliced += [self.get_points_transformed()[i] + segment_direction * j for j in linear]
+
+            # Signal progress update, handle interrupt (every 16 iterations to keep overhead low)
+            if i & 0xf == 0:
+                progress_callback(100 * (i + 1) / (len(self.get_points_transformed()) - 1))
+
+                if QThread.currentThread().isInterruptionRequested():
+                    Debug(self, ".recalculate(): Interruption requested, exiting now", color=Theme.PrimaryColor)
+                    return False
+
+        # Close the loop
+        points_sliced.append(self.get_points_transformed()[-1])
+
+        self._points_sliced = np.array(points_sliced)
+        self._length = length
+
+        return True
