@@ -26,6 +26,7 @@ from magneticalc.Groupbox import Groupbox
 from magneticalc.HLine import HLine
 from magneticalc.ModelAccess import ModelAccess
 from magneticalc.Table import Table
+from magneticalc.Theme import Theme
 from magneticalc.Wire import Wire
 
 
@@ -61,7 +62,11 @@ class Wire_Widget(Groupbox):
 
         Groupbox.__init__(self, "Wire")
 
-        points_icon_label = IconLabel("mdi.vector-square", "Points")
+        points_icon_label = IconLabel("mdi.vector-square", "Points", final_stretch=False)
+        table_shortcut_label = QLabel("  ⟨F2⟩   ⟨ESC⟩")
+        table_shortcut_label.setStyleSheet(f"font-size: 13px; color: {Theme.LightColor}")
+        points_icon_label.addWidget(table_shortcut_label)
+        points_icon_label.addStretch()
         table_add_button = QPushButton()
         table_add_button.setIcon(qta.icon("fa.plus"))
         points_icon_label.addWidget(table_add_button)
@@ -81,17 +86,12 @@ class Wire_Widget(Groupbox):
 
         table_total_layout = QHBoxLayout()
         table_total_label_left = QLabel("Total base points:")
-        table_total_label_left.setStyleSheet("""
-            color: #555555;
-            font-style: italic;
-        """)
-        self.base_total_label = QLabel("")
-        self.base_total_label.setStyleSheet("""
-            color: #2a7db0;
-        """)
-        self.base_total_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
+        table_total_label_left.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
+        self.table_total_label = QLabel("")
+        self.table_total_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
+        self.table_total_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         table_total_layout.addWidget(table_total_label_left, alignment=Qt.AlignVCenter)
-        table_total_layout.addWidget(self.base_total_label, alignment=Qt.AlignVCenter)
+        table_total_layout.addWidget(self.table_total_label, alignment=Qt.AlignVCenter)
         self.addLayout(table_total_layout)
 
         # --------------------------------------------------------------------------------------------------------------
@@ -114,6 +114,7 @@ class Wire_Widget(Groupbox):
         stretch_layout = QHBoxLayout()
         for i in range(3):
             self.stretch_spinbox[i] = QDoubleSpinBox(self.gui)
+            self.stretch_spinbox[i].setLocale(self.gui.locale)
             self.stretch_spinbox[i].setMinimum(self.StretchMin)
             self.stretch_spinbox[i].setMaximum(self.StretchMax)
             self.stretch_spinbox[i].setSingleStep(self.StretchStep)
@@ -159,7 +160,8 @@ class Wire_Widget(Groupbox):
 
         # - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 
-        self.rotational_symmetry_radius_spinbox = QDoubleSpinBox()
+        self.rotational_symmetry_radius_spinbox = QDoubleSpinBox(self.gui)
+        self.rotational_symmetry_radius_spinbox.setLocale(self.gui.locale)
         self.rotational_symmetry_radius_spinbox.setMinimum(self.RotationalSymmetryRadiusMin)
         self.rotational_symmetry_radius_spinbox.setMaximum(self.RotationalSymmetryRadiusMax)
         self.rotational_symmetry_radius_spinbox.setSingleStep(self.RotationalSymmetryRadiusStep)
@@ -193,16 +195,24 @@ class Wire_Widget(Groupbox):
 
         self.addLayout(rotational_symmetry_layout)
 
+        total_length_layout = QHBoxLayout()
+        total_length_label_left = QLabel("Total wire length:")
+        total_length_label_left.setStyleSheet("font-style: italic;")
+        self.total_length_label = QLabel("N/A")
+        self.total_length_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
+        self.total_length_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
+        total_length_layout.addWidget(
+            total_length_label_left,
+            alignment=Qt.AlignVCenter
+        )
+        total_length_layout.addWidget(self.total_length_label, alignment=Qt.AlignVCenter)
+        self.addLayout(total_length_layout)
+
         rotational_symmetry_total_layout = QHBoxLayout()
         rotational_symmetry_total_label_left = QLabel("Total transformed points:")
-        rotational_symmetry_total_label_left.setStyleSheet("""
-            color: #555555;
-            font-style: italic;
-        """)
+        rotational_symmetry_total_label_left.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
         self.transformed_total_label = QLabel("")
-        self.transformed_total_label.setStyleSheet("""
-            color: #2a7db0;
-        """)
+        self.transformed_total_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
         self.transformed_total_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         rotational_symmetry_total_layout.addWidget(
             rotational_symmetry_total_label_left,
@@ -211,12 +221,21 @@ class Wire_Widget(Groupbox):
         rotational_symmetry_total_layout.addWidget(self.transformed_total_label, alignment=Qt.AlignVCenter)
         self.addLayout(rotational_symmetry_total_layout)
 
+        replace_base_points_button = QPushButton()
+        replace_base_points_button.setIcon(qta.icon("mdi.content-copy"))
+        replace_base_points_button.setText("Replace base points")
+        replace_base_points_button.clicked.connect(
+            lambda: self.set_wire(points=self.gui.model.wire.get_points_transformed())
+        )
+        self.addWidget(replace_base_points_button)
+
         # --------------------------------------------------------------------------------------------------------------
 
         self.addWidget(HLine())
 
         self.addWidget(IconLabel("mdi.box-cutter", "Slicer Limit"))
-        slicer_limit_spinbox = QDoubleSpinBox()
+        slicer_limit_spinbox = QDoubleSpinBox(self.gui)
+        slicer_limit_spinbox.setLocale(self.gui.locale)
         slicer_limit_spinbox.setMinimum(self.SlicerLimitMinimum)
         slicer_limit_spinbox.setMaximum(self.SlicerLimitMaximum)
         slicer_limit_spinbox.setSingleStep(self.SlicerLimitStep)
@@ -234,14 +253,9 @@ class Wire_Widget(Groupbox):
 
         sliced_total_layout = QHBoxLayout()
         sliced_total_label_left = QLabel("Total sliced points:")
-        sliced_total_label_left.setStyleSheet("""
-            color: #555555;
-            font-style: italic;
-        """)
+        sliced_total_label_left.setStyleSheet(f"color: {Theme.LightColor}; font-style: italic;")
         self.sliced_total_label = QLabel("")
-        self.sliced_total_label.setStyleSheet("""
-            color: #2a7db0;
-        """)
+        self.sliced_total_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
         self.sliced_total_label.setAlignment(Qt.AlignRight | Qt.AlignVCenter)
         sliced_total_layout.addWidget(sliced_total_label_left, alignment=Qt.AlignVCenter)
         sliced_total_layout.addWidget(self.sliced_total_label, alignment=Qt.AlignVCenter)
@@ -251,7 +265,8 @@ class Wire_Widget(Groupbox):
 
         self.addWidget(HLine())
 
-        dc_spinbox = QDoubleSpinBox()
+        dc_spinbox = QDoubleSpinBox(self.gui)
+        dc_spinbox.setLocale(self.gui.locale)
         self.addWidget(IconLabel("fa.cog", "Wire Current"))
         dc_spinbox.setMinimum(self.DcMinimum)
         dc_spinbox.setMaximum(self.DcMaximum)
@@ -267,7 +282,7 @@ class Wire_Widget(Groupbox):
         self.addLayout(dc_layout)
 
         # Initially load wire from configuration
-        self.set_wire(recalculate=False, readjust_sampling_volume=False)
+        self.set_wire(recalculate=False, readjust_sampling_volume=False, invalidate_self=False)
 
     # ------------------------------------------------------------------------------------------------------------------
 
@@ -279,7 +294,8 @@ class Wire_Widget(Groupbox):
             slicer_limit=None,
             dc=None,
             recalculate=True,
-            readjust_sampling_volume=True
+            readjust_sampling_volume=True,
+            invalidate_self=True
     ):
         """
         Sets the wire. This will overwrite the currently set wire in the model.
@@ -292,6 +308,7 @@ class Wire_Widget(Groupbox):
         @param dc: DC value (float)
         @param recalculate: Enable to trigger final re-calculation (boolean)
         @param readjust_sampling_volume: Enable to readjust sampling volume
+        @param invalidate_self: Enable to invalidate the old wire before setting a new one
         """
         with ModelAccess(self.gui, recalculate):
 
@@ -300,20 +317,11 @@ class Wire_Widget(Groupbox):
             slicer_limit = self.gui.config.set_get_float("wire_slicer_limit", slicer_limit)
             dc = self.gui.config.set_get_float("wire_dc", dc)
 
-            # ToDo: Implement some kind of "self.gui.config.set_get_dict" to make this shorter
-            if rotational_symmetry is None:
-                count = self.gui.config.get_int("rotational_symmetry_count")
-                radius = self.gui.config.get_float("rotational_symmetry_radius")
-                axis = self.gui.config.get_int("rotational_symmetry_axis")
-            else:
-                count = self.gui.config.set_get_int("rotational_symmetry_count", rotational_symmetry["count"])
-                radius = self.gui.config.set_get_float("rotational_symmetry_radius", rotational_symmetry["radius"])
-                axis = self.gui.config.set_get_int("rotational_symmetry_axis", rotational_symmetry["axis"])
-            rotational_symmetry = {
-                "count": count,
-                "radius": radius,
-                "axis": axis
-            }
+            rotational_symmetry = self.gui.config.set_get_dict(
+                prefix="rotational_symmetry_",
+                types={"count": "int", "radius": "float", "axis": "int"},
+                values=rotational_symmetry,
+            )
 
             self.gui.model.set_wire(
                 Wire(
@@ -322,7 +330,8 @@ class Wire_Widget(Groupbox):
                     rotational_symmetry=rotational_symmetry,
                     slicer_limit=slicer_limit,
                     dc=dc
-                )
+                ),
+                invalidate_self=invalidate_self
             )
 
             self.update_table()
@@ -382,13 +391,14 @@ class Wire_Widget(Groupbox):
 
         self.table.set_contents(points)
 
-        self.base_total_label.setText(str(len(self.gui.model.wire.get_points_base())))
+        self.table_total_label.setText(str(len(self.gui.model.wire.get_points_base())))
 
     def on_table_row_added(self):
         """
-        Adds a wire base point after a row has been added to the table.
+        Adds a wire base point after a row has been added to the table. (Appending to the end and focusing the row.)
         """
-        self.set_wire(points=[np.array([0, 0, 0])] + list(self.gui.model.wire.get_points_base()))
+        self.set_wire(points=list(self.gui.model.wire.get_points_base()) + [np.zeros(3)])
+        self.table.select_last_row()
 
     def on_table_cell_edited(self, item):
         """
@@ -422,5 +432,9 @@ class Wire_Widget(Groupbox):
         """
         if self.gui.model.wire.is_valid():
             self.sliced_total_label.setText(str(len(self.gui.model.wire.get_points_sliced())))
+            self.total_length_label.setText(f"{self.gui.model.wire.get_length():.2f} cm")
+            self.total_length_label.setStyleSheet(f"color: {Theme.PrimaryColor}; font-weight: bold;")
         else:
             self.sliced_total_label.setText("N/A")
+            self.total_length_label.setText("N/A")
+            self.total_length_label.setStyleSheet(f"color: {Theme.PrimaryColor};")
